import { Result } from "jalhyd";

import { CalculatedParamResults } from "./param-calc-results";
import { NgParameter } from "../formulaire/ngparam";
import { ServiceFactory } from "../services/service-factory";

export class PabResults extends CalculatedParamResults {

    /** résultats des modules Cloisons avant chaque bassin */
    public cloisonsResults: Result[];

    /** résultats du module ParallelStructure pour la cloison aval */
    public cloisonAvalResults: Result;

    /** cote aval de l'ensemble de la passe */
    public Z2: number;

    /** paramètres variés */
    public variatedParameters: NgParameter[];

    /** index de la valeur du paramètre varié à afficher dans les résultats */
    public variableIndex = 0;

    /** symboles des colonnes de résultat */
    protected _columns: string[];

    public constructor() {
        super();
        this.reset();
        // standard headers
        this._columns = [
            "CLOISON",
            "Z",
            "ZRAM",
            "DH",
            "Q",
            "PV",
            "YMOY",
            "ZRMB",
            "QA"
        ];
    }

    /** headers symbols */
    public get columns() {
        return this._columns;
    }

    /** translated headers texts */
    public get headers() {
        return this._columns.map((h) => {
            // calculator type for translation
            const sn = this.result.sourceNub;
            let ct = sn.calcType;
            if (sn.parent) {
                ct = sn.parent.calcType;
            }
            return ServiceFactory.instance.formulaireService.expandVariableNameAndUnit(ct , h);
        });
    }

    public reset() {
        super.reset();
        this.cloisonsResults = [];
        this.cloisonAvalResults = undefined;
        this.Z2 = undefined;
    }

    /** retourne true si au moins un calcul a échoué (le log a un code négatif) */
    public hasError(): boolean {
        let err = false;
        // log principal
        err = (err || this.result.hasErrorMessages);
        // logs des cloisons
        for (const c of this.cloisonsResults) {
            err = (err || c.hasErrorMessages);
        }
        // log de la cloison aval
        err = (err || this.cloisonAvalResults.hasErrorMessages);

        return err;
    }
}
